function [HAR_DJIA] = AR_explanatory_var(series,lags,lags_square,intercept,X_expl)
%%%% function that creates the explanatory variables of an AR models
%%%%%%%%%%%%%%%%
%%%% Inputs: 
%%%% series: Tx1 stands for the dependent variable
%%%% lags: vector indicating which lags you want (e.g. [1 2]: AR(2) model)
%%%% lags_square: vector indicating which squared lagged variables you want
%%%% (if none: lags_square = [];)
%%%% intercept: =1 if you want to include an intercept
%%%% X_expl: additional explanatory variables (must by TxK in which K
%%%% denotes the number of additionnal variables).
%%%%%%%%%%%%%%%%
%%%% Output: A structure with the dependent and the explanatory variables.
if(nargin<5)
    X_expl = [];
    if(nargin<4)
        intercept = 0;
        if(nargin<3)
            lags_square = [];
            if(nargin<2)
                lags = [1 2];
            end
        end
    end
end

nb_AR = max(size(lags));
nb_AR_square = max(size(lags_square));
taille_AR = nb_AR+nb_AR_square;
y = series;
taille = max(size(y));
if(intercept~=0)
    intercept = 1;
end

dimension_X = 0;
if(max(size(X_expl))==max(size(series)))
    if(size(X_expl,1)~=max(size(series)))
        X_expl = X_expl';
    end
    dimension_X = size(X_expl,2);
end

X = ones(taille,taille_AR + intercept + dimension_X);
if(dimension_X~=0)
    X(:,taille_AR + intercept+1:end) = X_expl;
end
moyenne = 10;
if(moyenne>taille)
    moyenne = taille;
end
for t=1:taille
    for j=1:nb_AR
        if(t<=lags(j))
            X(t,intercept+j) = mean(series(1:moyenne));
        else
            X(t,intercept+j) = series(t-lags(j));
        end
    end
    for j=1:nb_AR_square
        if(t<=lags(j))
            X(t,intercept+nb_AR+j) = mean(series(1:moyenne))^2;
        else
            X(t,intercept+nb_AR+j) = series(t-lags_square(j))^2;
        end
    end
end
HAR_DJIA.y = y;
HAR_DJIA.X = X;  