function [DGP] = DGP_simulation(type_model,type_simul)
%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Simulation of series from Data generating processes
%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Inputs 
%%% type_model: 'AR', 'ARMA' or 'GARCH'
%%% type_simul: DGP specification (see below)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% For AR models, possible DGP specifications: {'A','B','C','D','E','F','no_break'}
%%% For ARMA models, possible DGP specifications: {'G','H','no_break_MA'}
%%% For GARCH models, possible DGP specifications: {'DGP11','DGP12','A','B','C','D','E','F','G','H','J','no_break'}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Output: a structure containing all the DGP specification and a
%%% simulated series
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(nargin<1)
    type_model = 'AR';
    if(nargin<2)
        type_simul = 'no_break';
    end
end
switch type_model
    case {'AR','ARMA'}
        addpath('./Estimation_AR');
        ARMA_type = {'G';'H';'no_break_MA';'I';'J'};
        
        switch type_model
            case 'ARMA'
                if(strcmp(type_simul,ARMA_type)==0)
                   warning('type of simul does not correspond to an ARMA model')
                   type_simul = 'no_break_MA';
                end
            case 'AR'
                if(strcmp(type_simul,ARMA_type)==1)
                   warning('type of simul does not correspond to an AR model')
                   type_simul = 'no_break';
                end
        end
        [DGP] = DGP_JRSSB(type_simul);
    case 'GARCH'
        addpath('./Estimation_GARCH');
        [DGP] = DGP_GARCH_simulation(type_simul);
end
DGP.type_model = type_model;