/*********************************************************************
 * Demo.cpp
 *
 * This file shows the basics of setting up a mex file to work with
 * Matlab.  This example shows how to use 2D matricies.  This may
 * 
 * Keep in mind:
 * <> Use 0-based indexing as always in C or C++
 * <> Indexing is column-based as in Matlab (not row-based as in C)
 * <> Use linear indexing.  [x*dimy+y] instead of [x][y]
 *
 * For more information, see my site: www.shawnlankton.com
 * by: Shawn Lankton
 *
 ********************************************************************/
#include <matrix.h>
#include <mex.h>   
#include <math.h>
#include <float.h>

/* Definitions to keep compatibility with earlier versions of ML */
#ifndef MWSIZE_MAX
typedef int mwSize;
typedef int mwIndex;
typedef int mwSignedIndex;

#if (defined(_LP64) || defined(_WIN64)) && !defined(MX_COMPAT_32)
/* Currently 2^48 based on hardware limitations */
# define MWSIZE_MAX    281474976710655UL
# define MWINDEX_MAX   281474976710655UL
# define MWSINDEX_MAX  281474976710655L
# define MWSINDEX_MIN -281474976710655L
#else
# define MWSIZE_MAX    2147483647UL
# define MWINDEX_MAX   2147483647UL
# define MWSINDEX_MAX  2147483647L
# define MWSINDEX_MIN -2147483647L
#endif
#define MWSIZE_MIN    0UL
#define MWINDEX_MIN   0UL
#endif
/*  Fonction pour la densite d'un modele ARMA : 8 input : la serie y (T*1), les parametres GARCH, paramtres mu, cass , eta 
    L'output : la densite et la variance conditionnelle
 *[log_dens sigma] = dens_GJR_GARCH_MCMC(y,theta_GARCH,theta_mu,cass,eta)
 *[log_dens X] = dens_CP_ARMA_c(y,X,taille_totale_X,MA_lags,theta,sigma,tau)
 *
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

//declare variables
    mxArray *a_in_m, *b_in_m, *c_in_m, *d_in_m, *e_in_m,*f_in_m,*g_in_m,*h_in_m, *e_out_m,*theta_aide,*mu_aide;
    const mwSize *dims;
    double *y, *log_like,*theta,*X,*sigma,*tau,*theta_save;//,*AR_lags,*MA_lags;
    int dimx, dimy, numdims;
    int i,q,z,j,taille,ind,index,index_prev,ind_mu,taille_ARMA,taille_totale_X,MA_lags,regime;
    double pi,log_dens,eps,eps_sq,*X_out;
    
    if (nrhs > 7) 
    {
        mexErrMsgTxt("7 inputs required.");
    } 


//associate inputs
    a_in_m = mxDuplicateArray(prhs[0]);
    b_in_m = mxDuplicateArray(prhs[1]);
    c_in_m = mxDuplicateArray(prhs[2]);
    d_in_m = mxDuplicateArray(prhs[3]);
    e_in_m = mxDuplicateArray(prhs[4]);
    f_in_m = mxDuplicateArray(prhs[5]);
    g_in_m = mxDuplicateArray(prhs[6]);

    

//figure out dimensions
    dims = mxGetDimensions(prhs[0]);
    numdims = mxGetNumberOfDimensions(prhs[0]);
    dimy = (int)dims[0]; dimx = (int)dims[1];
    taille = dimy;
    dims = mxGetDimensions(prhs[5]);
    regime = (int)dims[0];
 

    #define f_out_m  plhs[1]
    f_out_m = mxCreateDoubleMatrix(0,0,mxREAL);
    mxSetM(f_out_m,taille);
    mxSetN(f_out_m,1);
    mxSetData(f_out_m,mxMalloc(sizeof(double)*taille));
    X_out = mxGetPr(f_out_m);
    
//associate outputs
    //log_like = plhs[0] = mxCreateDoubleMatrix(dimy,dimx,mxREAL);
    e_out_m = plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
    

//associate pointers =  dens_CP_ARMA(y,X,AR_lags,MA_lags,theta,sigma,tau)
    y = mxGetPr(a_in_m);
    X = mxGetPr(b_in_m);
    taille_ARMA = (int) (*mxGetPr(c_in_m));
    MA_lags = (int) (*mxGetPr(d_in_m));
    theta =  mxGetPr(e_in_m);
    sigma =  mxGetPr(f_in_m);
    tau =  mxGetPr(g_in_m);

    
//     if(nlhs == 8) 
//     {
//       rec_var = mxGetPr(h_in_m);
//     }
//     else
//     {
//         rec_var = sigma;
//         for(q=0;q<regime;q++)
//         {
//             rec_var[q] = 0;
//         }
//     }
    
    //taille_ARMA = 1+AR_lags+MA_lags;
    
//      mexPrintf("taille_ARMA = %d\n",taille_ARMA);    
//      mexPrintf("tau1 = %d\n",tau[0]);
//      mexPrintf("sigma1 = %f\n",sigma[0]);    
//      
//      
//      mexPrintf("X 0 = %f\n",X[0]);    
//      mexPrintf("X 1 = %f\n",X[1]);    
//      mexPrintf("X taille+1 = %f\n",X[taille+1]);    
//      mexPrintf("X taille+2 = %f\n",X[taille+2]); 
    
//     mexPrintf("theta_all[%d] = %f\n",0,theta_all[0]);
//     
//     mexPrintf("dimy = %d\n",dimy);
//     mexPrintf("dimx = %d\n",dimx);
    
//     mxDestroyArray(a_in_m);
//     mxDestroyArray(b_in_m);
//     mxDestroyArray(c_in_m);
//     mxDestroyArray(d_in_m);
    
    log_like = mxGetPr(e_out_m);
    
    
//     for(q=0;q<regime;q++)
//     {
//       index = (q)*taille_ARMA;  
//       for(z=0;z<taille_ARMA;z++)
//       {
//           mexPrintf("theta[%d] = %f\n",index+z,theta[index+z]);            
//       }  
//     }
    
 

    
    pi = 3.14159265;
    
    ind = 1;
    index = (ind-1)*taille_ARMA;
//     if(theta[index+2]>1)
//     {
//         log_dens = -1e10;
//         
//     }
//     else
//     {
        eps = 0;
        for(q=0;q<taille_ARMA;q++)
        {
          eps = eps + X[q*taille+0]*theta[index+q];  
        }
        eps = y[0]-eps;
        if(MA_lags>0)
        {
            X[(taille_ARMA-MA_lags)*taille+1] = eps;
           
        }
         X_out[0] = eps;
        eps_sq= eps*eps;
        log_dens = -0.5*log(2*pi*sigma[0])-0.5*eps_sq/sigma[0];
//             mexPrintf("log_dens = %f\n",log_dens);
//             mexPrintf("theta[%d] = %f\n",0,theta[0]);
//             mexPrintf("theta[%d] = %f\n",1,theta[1]);
//             mexPrintf("theta[%d] = %f\n",2,theta[2]);
//             mexPrintf("theta[%d] = %f\n",3,theta[3]);
// //            mexPrintf("sigma_sq[%d] = %f\n",0,sigma_sq[0]);
//            mexPrintf("eps = %f\n",eps);
//            mexPrintf("y = %f\n",y[0]);
// //           mexPrintf("mu = %f\n",mu[ind-1]);
//            mexPrintf("log_dens = %f\n",log_dens);
//          system("pause");

    //do something
        ind_mu = 0;
        for(i=1;i<taille;i++)
        {
            if(i==tau[ind-1])
            {
                ind = ind+1;
                ind_mu = ind-1;//eta[ind-1]-1;
                //mexPrintf("iteration [%d] = %d, ind_mu : %d\n",i,ind,ind_mu);
            }
            index = (ind-1)*taille_ARMA;
            
            
            eps = 0;
            for(q=0;q<taille_ARMA;q++) {
                eps = eps + X[q*taille+i]*theta[index+q];
            }
            eps = y[i]-eps;
            X_out[i] = eps;
            if(i+1<taille && MA_lags>0) 
            {
                X[(taille_ARMA-MA_lags)*taille+i+1] = eps;
                if(MA_lags>1)
                {
                    for(j=2;j<=MA_lags;j++)
                    {
                        if(i+2-j>0)
                        {
                            X[(taille_ARMA-MA_lags+j-1)*taille+i+1] = X[(taille_ARMA-MA_lags)*taille+i+2-j];
                            
                        }
                    }
                }
            }
            
            eps_sq= eps*eps;
            log_dens = log_dens -0.5*log(2*pi*sigma[ind_mu])-0.5*eps_sq/sigma[ind_mu];
            
//             if(sigma_sq[i]<0)
//             {
//                 mexPrintf("sigma[%d] = %f\n",i,sigma[ind_mu]);
//              mexPrintf("log_dens[%d] = %f\n",i,log_dens);
//             mexPrintf("log_dens[%d] = %d\n",i,index);
//             mexPrintf("theta[%d] = %f\n",index+0,theta[index+0]);
//             mexPrintf("theta[%d] = %f\n",index+1,theta[index+1]);
//             mexPrintf("theta[%d] = %f\n",index+2,theta[index+2]);
//             mexPrintf("theta[%d] = %f\n",index+3,theta[index+3]);
//             }
        }
        
        if(log_dens<-1e17 || log_dens>1e17)
        {
            log_dens = -1e17;
        }

        /* Pour le compilage sur visual c++ : include float.h et c'est la fonction _isnan
         if(_isnan(log_dens)==1)
         {
             log_dens = -1e17;
         }
        */
        /* Pour le compilage sur SDK : include math.h et c'est la fonction isnan*/
         if(isnan(log_dens)==1)
         {
             log_dens = -1e17;
         }
         


//          if(isnan(log_dens)==1 || isinf(log_dens)==1)
//          {
//              log_dens = -1e10;
//          }
        
//     }
    log_like[0] = log_dens;
    
    
    

    return;
}
