function [MLE] = max_ARMA(y_AR,X_AR,MA_lags,MLE_prev)
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Find the maximum likelihood estimate of an ARMA(AR,MA) model with
%%%% MA=MA_lags (MA_lags>0) and X_AR of dimension 1+AR+MA which includes an
%%%% intercept, the AR terms and a place for the MA terms.
%%%%%%%%%%%%%%%%%%%%%%%%%
starting_point = 0;
if(nargin==4)
    starting_point = 1;
end

AR_lags = min(size(X_AR))-1;

%%%% iterative forecast
nb_forecast = 22*4;


taille = max(size(y_AR));
cass =  taille; %%% no break

X_AR = [X_AR zeros(taille,MA_lags)];
taille_ARMA = 1+AR_lags+MA_lags;
regime = max(size(cass));

option = optimset('MaxIter',1e7,'MaxFunEvals',1e7);
if(MA_lags==0)
    nb = 2;
else
    nb = 6;
end


spike_and_slab = 0;
var_prior_ARMA = 0.1;

for i=1:nb
    test = 0;
    while(test==0)
        if(MA_lags==0 && i==1)
            theta_current = (X_AR'*X_AR)\(X_AR'*y_AR);
        else
            theta_current = randn(regime*taille_ARMA,1)*sqrt(var_prior_ARMA);
        end
        sigma_current = randn(regime,1);
        if(i==1 && starting_point==1)
            sigma_current = MLE_prev.sigma;
            theta_current = MLE_prev.theta;
        end
        theta_all = [theta_current;sigma_current];
        log_dens = -1*MLE_likelihood_ARMAX(theta_all,cass,regime,taille_ARMA,MA_lags,y_AR,X_AR,spike_and_slab);
        if(log_dens<1e9)
            test = 1;
        end
    end
    
    [val_theta,log_dens,~,~,~,hessian] = fminunc(@(theta_all) -1*MLE_likelihood_ARMAX(theta_all,cass,regime,taille_ARMA,MA_lags,y_AR,X_AR,spike_and_slab),theta_all,option);
    log_dens = -1*log_dens; 
    if(i==1)
        max_dens = log_dens;
        max_theta = val_theta(1:regime*taille_ARMA);
        max_log_sigma = val_theta(regime*taille_ARMA+1:end);
        hessian_MLE = hessian;
        [~,theta_sorted,sigma_sorted,~,eps] = MLE_likelihood_ARMAX(val_theta,cass,regime,taille_ARMA,MA_lags,y_AR,X_AR,spike_and_slab);
    elseif(max_dens<log_dens)
        max_dens = log_dens;
        max_theta = val_theta(1:regime*taille_ARMA);
        max_log_sigma = val_theta(regime*taille_ARMA+1:end);
        hessian_MLE = hessian;
        [~,theta_sorted,sigma_sorted,~,eps] = MLE_likelihood_ARMAX(val_theta,cass,regime,taille_ARMA,MA_lags,y_AR,X_AR,spike_and_slab);
    end
    
    
    
end


MLE.variance = diag(inv(hessian_MLE));
MLE.std = sqrt(MLE.variance);

MLE.cass = cass;
MLE.theta = max_theta;
MLE.sigma = max_log_sigma;
MLE.theta_sorted = theta_sorted;
MLE.sigma_sorted = sigma_sorted;
MLE.log_like = max_dens;
MLE.X_AR = X_AR;
MLE.y_AR = y_AR;
MLE.eps = eps;
MLE.hessian = hessian_MLE;

ypred = zeros(nb_forecast,1);

X_term = zeros(1,taille_ARMA);
X_term(1) = 1;

if(AR_lags>0)
    X_term(2:2+AR_lags-1) = y_AR(taille:-1:taille-AR_lags+1);
end

if(MA_lags==0)
    MLE.OLS = (X_AR'*X_AR)\(X_AR'*y_AR);
    ypred(1) = X_term*MLE.theta_sorted;
else
   MLE.OLS = 0;
   for q=1:MA_lags
      X_term(end-MA_lags+q) = eps(end-q+1); 
   end
   ypred(1) = X_term*MLE.theta_sorted;
   X_term(end-MA_lags+1:end) = 0;
end


for t=2:nb_forecast
    if(AR_lags>1)
         X_term(3:3+AR_lags-2) = X_term(2:2+AR_lags-2);
    end
    if(AR_lags>0)
        X_term(2) = ypred(t-1);
    end
    ypred(t) = X_term*MLE.theta_sorted;
end
MLE.ypred = ypred;


%%%%%%%%%%%%%%%%% Test de Box-Pierce
lags = 50;
eps = MLE.eps;
MLE.test_Box = test_Box_Pierce(eps,lags);
MLE.test_Box_on_squared_error = test_Box_Pierce(eps.^2,lags);

%%%%%%%%%%%% Test Normalit
[test_JB.Reject_Normal,test_JB.p_value,test_JB.Q_stat,test_JB.critval] = jbtest(eps,0.05);
MLE.test_JB = test_JB;
MLE.y = y_AR;
MLE.X = X_AR;