function [DGP] = DGP_JRSSB(type_simul,T)
%%% Simulation from Autoregressive and ARMA DGPs.
%%%% AR/ARMA DGPs from 'Inference for multiple Change points in teime series via
%%%% likelihood ratio scan statistics (2016), JRSSB.
if(nargin<2)
    T = 1024;
end

if(nargin<1)
    type_simul = 'no_break';
end
switch type_simul
    case 'AR4'
        cass = [400 612 1024];
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        regime = max(size(cass));
        theta = [0 0.6 -0.3 -0.4 0.2;0 -0.6 -0.3 -0.4 0.2;0 0.5 -0.3 -0.4 0.2];
        sigma = ones(regime,1);
        nb_AR = 4;
    case 'A'
        cass = 1024;
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
        theta = [0 -0.7];
        sigma = ones(regime,1);
        nb_AR = 1;
    case 'B'
        cass = [512 768 1024];
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
        theta = [0 0.9 0;0 1.69 -0.81;0 1.32 -0.81];
        sigma = ones(regime,1);
        nb_AR = 2;
    case 'C' %%% Similar ACF starting from the second lag.
        cass = [400 612 1024];
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
        theta = [0 0.4;0 -0.6;0 0.5];
        sigma = ones(regime,1);
        nb_AR = 1;
    case 'D'
        cass = [50 1024];
        
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
        theta = [0 0.75;0 -0.5];
        sigma = ones(regime,1);
        nb_AR = 1;
    case 'E'
        cass = [400 750 1024];
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
        theta = [0 0.999;0 0.999;0 0.999];
        sigma = ones(regime,1);
        sigma(2) = 1.5^2;
        nb_AR = 1;
    case 'F' %%% Same ACF between regimes
        cass = [400 750 1024];
        
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
        theta = [0 1.399 -0.4;0 0.999 0;0 0.699 0.3];
        sigma = ones(regime,1);
        sigma(2) = 1.5^2;
        nb_AR = 2;
    case 'G'
        cass = [512 768 1024];
        
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
%         theta = [0 -0.9 0.7;0 0.9 -0.7;0 0 -0.7];
        theta = [0 0.9 0.7;0 -0.9 -0.7;0 0 -0.7];
        sigma = ones(regime,1);
        nb_AR = 1;
    case 'H'
        cass = [512 768 1024];
        
        lambda = cass/cass(end);
        cass = ceil(lambda*T);
        
        regime = max(size(cass));
        theta = [0 0.9 0.7;0 0.9 -0.7;0 0.9 -0.7];
        sigma = ones(regime,1);
        sigma(3) = 2.5;
        nb_AR = 1;
        
    case 'no_break_MA'
        cass = 500;
        regime = max(size(cass));
        theta = [0.5 0.8 0.4];
        sigma = ones(regime,1);
        nb_AR = 1;        
    otherwise %%% No break
        cass = 2000;
        regime = max(size(cass));
        theta = [0.5 0.8];
        sigma = ones(regime,1);
        nb_AR = 1;
end
regime = max(size(cass));

switch type_simul
    case {'G','H','no_break_MA'}
        nb_MA = 1;
    case 'I'
        nb_MA = 2;
    otherwise
        nb_MA = 0;
end
nb_lags = max([nb_AR nb_MA]);
T = cass(end);
burn = 100;
T_all = T+burn;
epsilon = zeros(T_all,1);
y = zeros(T_all,1);
X = ones(nb_AR+1,1);
epsilon(1:nb_lags) = randn(nb_lags,1)*sqrt(sigma(1));
y(1:nb_lags) = epsilon(1:nb_lags);
cass = cass + burn;
ind = 1;
for t=nb_lags+1:T_all
    for z=1:nb_AR
       X(z+1) = y(t-z); 
    end
    if(t>cass(ind))
        ind = ind +1;
    end
    epsilon(t) =  randn()*sqrt(sigma(ind));
    if(nb_MA~=0)
        y(t) = theta(ind,1:nb_AR+1)*X + theta(ind,nb_AR+2:end)*epsilon(t-1:-1:t-nb_MA) +  epsilon(t);
    else
        y(t) = theta(ind,1:nb_AR+1)*X + epsilon(t);
    end
end
DGP.cass = cass-burn;
DGP.regime = regime;
DGP.theta = theta;
DGP.sigma = sigma;
DGP.nb_AR = nb_AR;
DGP.nb_MA = nb_MA;
DGP.y = y(burn+1:end);
DGP.epsilon = epsilon(burn+1:end);
DGP.type_simul = type_simul;
if(regime>1)
    DGP.nb_regime_per_param = [(1+sum(diff(theta,1)~=0,1)) 1+sum(diff(sigma)~=0)]';
else
    DGP.nb_regime_per_param = ones(2+nb_AR+nb_MA,1);
end
break_dates = [1 cass];
nb_acf = 30;
emp_acf = zeros(nb_acf,regime);
for q=1:regime
   [acf] = autocorr(y(break_dates(q):break_dates(q+1)),nb_acf);
   emp_acf(:,q) = acf(2:end);
end
DGP.emp_acf = emp_acf;