function [result] = DE_optim_GARCH(y,break_dates,N,nb_iter,mat_const)
%%%%%%
%%% Optimization of a GARCH model using a Differential Evolution algorithm.
regime = max(size(break_dates));
if(nargin<5)
    mat_const = zeros(3,regime);
    if(nargin<4)
        nb_iter = 150;
    end
end

warning off all;
fval = zeros(N,1);
%%%%% Initialisation
unc_var = var(y);
alpha = 0.1;
beta = 0.85;
log_theta_start = [log(unc_var) log(alpha/(beta+alpha-alpha)) log(beta/(1-beta))];


log_theta_start = kron(ones(1,regime),log_theta_start);
dim_model = sum(sum(mat_const==0));%3*regime;
if(dim_model==3*regime)
    constraint = 0;
else
    constraint = mat_const;
end
particles = zeros(N,dim_model);
option = optimset('MaxIter',20,'MaxFunEvals',50,'display','off');
mat_const_vect = reshape(mat_const,regime*3,1);
for q=1:N
    if(q==1)
        log_theta = log_theta_start + randn(1,3*regime)*0.1;
    else
        log_theta = log_theta_start + randn(1,3*regime);
    end
    log_theta = log_theta(mat_const_vect==0);
    particles(q,:) = log_theta;
    [val_theta,like] = fminunc(@(log_theta) -1*log_likelihood_GARCH_with_break_dates(y,log_theta,break_dates,constraint),log_theta,option);
    particles(q,:) = val_theta;
   [fval(q)] = -1*like;%log_likelihood_GARCH_with_break_dates(y,particles(q,:),break_dates);
end

[gbest,ind] = max(fval);
particle_best = particles(ind,:);

CR = 0.3;
%%%%% Differential Evolution
for i=1:nb_iter
    for q=1:N
        F = 0.4 + rand()*(1-0.4);
        if(rand()>0.3)
            ind = ceil(rand()*N);
            ind2 = ceil(rand()*N);
            u = particles(q,:) + F*(particles(ind,:)-particles(ind2,:));
        else
            ind = ceil(rand()*N);
            u = particles(q,:) + F*(particle_best-particles(ind,:));
        end
        
        accept = rand(1,3)>CR;
        log_theta_prop = particles(q,:);
        log_theta_prop(accept) = u(accept);
        [fval_prop] = log_likelihood_GARCH_with_break_dates(y,log_theta_prop,break_dates,constraint);
        
        if(fval_prop>fval(q))
            particles(q,:) = log_theta_prop;
            fval(q) = fval_prop;
            if(fval_prop>gbest)
                gbest = fval_prop;
                particle_best = log_theta_prop;
            end
        end
    end
    
end
[gbest,theta_best,sigma] = log_likelihood_GARCH_with_break_dates(y,particle_best,break_dates,constraint);
result.sigma_best = sigma;
result.particles_best = particle_best;
result.theta_best = theta_best;
result.bestfit = gbest;
result.particles = particles;
result.fval = fval;