function [DGP] = DGP_GARCH_simulation(type_simul,T)
%%%% GARCH DGPs in 'Inference for Multiple Change-points in Linear and Non-Linear TS models' (2017)
if(nargin<2)
    T = 4000;
    if(nargin<1)
        type_simul = 'no_break';
    end
end
switch type_simul
    case {'DGP11'}
        theta = [3 0.1 0.5;1 0.1 0.5;1 0.4 0.9];
    case {'DGP12'}
        theta = [3 0.1 0.9;1 0.1 0.9;1 0.4 0.9];
    case 'A'
        theta = [0.4 0.1 0.5;0.4 0.1 0.5];
    case 'C'
        theta = [0.4 0.1 0.5;0.4 0.1 0.6];
    case 'E'
        theta = [0.1 0.1 0.8;0.1 0.1 0.7];
    case 'G'
        theta = [0.4 0.1 0.5;0.5 0.1 0.5];
    case 'I'
        theta = [0.1 0.1 0.8;0.3 0.1 0.8];
    case 'B'
        theta = [0.1 0.1 0.8;0.1 0.1 0.8];
    case 'D'
        theta = [0.4 0.1 0.5;0.4 0.1 0.8];
    case 'F'
        theta = [0.1 0.1 0.8;0.1 0.1 0.4];
    case 'H'
        theta = [0.4 0.1 0.5;0.8 0.1 0.5];
    case 'J'
        theta = [0.1 0.1 0.8;0.5 0.1 0.8];
    otherwise %%% no break
        theta = [0.1 0.1 0.85];
end

switch type_simul
    case {'DGP11';'DGP12'}
       cass = round([1250 2000 4000]*T/4000);
    case {'A','B','C','D','E','F','G','H','J'}
        cass = round([0.5 1]*T);
    otherwise
        cass = 2000;
end
regime = max(size(cass));
if(size(theta,1)~=regime)
    error('DGP problem')
end
T = cass(end);
burn = 200;
T_all = T + burn;
sigma_t = zeros(T_all,1);
y = zeros(T_all,1);
cass = cass + burn;
ind = 1;

unc_var = theta(ind,1);
sigma_t(1) = unc_var;
y(1) = randn()*sqrt(sigma_t(1));
for t=2:T_all
    if(t>cass(ind))
        ind = ind +1;
    end
    X_t = [1 (y(t-1)^2-sigma_t(t-1)) (sigma_t(t-1)-theta(ind,1))]';
   sigma_t(t) = theta(ind,:)*X_t;
    y(t) = randn()*sqrt(sigma_t(t)); 
end

DGP.y = y(burn+1:end);
DGP.sigma_t = sigma_t(burn+1:end);
DGP.theta = theta;
DGP.cass = cass-burn;
DGP.type_simul = type_simul;
if(regime>1)
    DGP.nb_regime_per_param = (1+sum(diff(theta,1)~=0,1))';
else
    DGP.nb_regime_per_param = ones(3,1);
end
mu = zeros(regime,1);
theta_current = reshape(theta',regime*3,1);
DGP.log_like = dens_CP_GARCH_MCMC(DGP.y,theta_current,mu,DGP.cass,theta_current(1));


