function [Simu] = MLE_GARCH_with_known_break_dates(y,break_dates,nb_repet,choice_constraint)
%%%% Maximum likelihood estimation of a GARCH(2,2) model given the
%%%% breakpoints.
if(nargin<4)
    choice_constraint = 'none';
    if(nargin<3)
        nb_repet=1;
    end
end


dim_y = size(y);
taille = size(y,1);
if(taille~=max(dim_y))
    y = y';
end
if(taille~=break_dates(end))
    error('Break dates mispecified')
end

regime = max(size(break_dates));
switch choice_constraint
    case 'DGP11'
        if(regime~=3)
            warning('No constraint');
            mat_const = zeros(3,regime);
        else
            mat_const = [0 0 1;0 1 0;0 1 0];
        end
    case 'cDGP11'
        if(regime~=3)
            warning('No constraint');
            mat_const = zeros(3,regime);
        else
            mat_const = [0 0 1;0 1 0;0 1 1];
        end
    otherwise
        mat_const = zeros(3,regime);
end
nb_param = sum(sum(mat_const==0));
if(nb_param==3*regime)
    constraint = 0;
else
    constraint = mat_const;
end
mat_const_vect = reshape(mat_const,regime*3,1);
option = optimset('MaxIter',1e7,'MaxFunEvals',1e7,'display','off');
MLE_like = -1e10;
for i=1:nb_repet
    if(regime==1)
        [result] = DE_optim_GARCH(y,break_dates,30,mat_const);
    else
        [result] = DE_optim_GARCH(y,break_dates,30,mat_const);
    end
    log_theta_start = result.particles_best(mat_const_vect==0);
    [val_theta,like,~,~,~,hessian] = fminunc(@(log_theta) -1*log_likelihood_GARCH_with_break_dates(y,log_theta,break_dates,constraint),log_theta_start,option);
   if(i==1)
       MLE_theta = val_theta;
       MLE_like = like;
       MLE_hessian = hessian;
   elseif(like>MLE_like)
       MLE_theta = val_theta;
       MLE_like = like;
       MLE_hessian = hessian;
   end 
end
[Simu.log_like,Simu.theta,Simu.sigma] = log_likelihood_GARCH_with_break_dates(y,MLE_theta,break_dates,constraint);
alpha = Simu.theta(2);
beta = Simu.theta(3);
nb_corr = 200;
corr_th = zeros(nb_corr,1);
num = (1-beta^2-alpha*beta);
den = (1-beta^2-2*alpha*beta);
corr_th(1) = alpha*num/den;
for t=2:nb_corr
    corr_th(t) = (alpha+beta)*corr_th(t-1);
end
Simu.corr_th = corr_th;
Simu.MLE_hessian = MLE_hessian;
Simu.std_param = sqrt(diag(inv(Simu.MLE_hessian)));

