%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% How to run the codes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Main program: SMC_CP_in_TS.m
%%% See the documentation of SMC_CP_in_TS.m for more information.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% How to run AR models ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear arg_model;
[DGP] = DGP_simulation('AR','B'); %%% Simulation of a series from DGP 2 of the paper.
arg_model.AR_lags = 2; %% arg_model contains information about the model specification
[Summary,Simu] = SMC_CP_in_TS(DGP.y,'AR',arg_model,5000); %%% Estimation of a Sparse CP-AR(2) model

disp('True number of regimes for the intercept, AR terms and the variance:')
disp(DGP.nb_regime_per_param');
disp('Number of detected regimes:')
disp(Summary.nb_regime_per_param');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% How to include other explanatory variables ? 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear arg_model;
[DGP] = DGP_simulation('AR','B'); %%% Simulation of a series from DGP 2 of the paper.
[DATA] = AR_explanatory_var(DGP.y,[4 5],[],0,[]); %%% generates the explanatory variables of an AR([4 5]) without intercept
y = DATA.y; %%% dependent variable
X = DATA.X; %%% Explanatory variables
arg_model.AR_lags = 2;
arg_model.X = X;
[Summary,Simu] = SMC_CP_in_TS(y,'AR',arg_model,5000); %%% Estimation of a Sparse CP-AR([1 2 4 5]) model with intercept

disp('True number of regimes for the intercept, AR terms and the variance:')
disp(DGP.nb_regime_per_param');
disp('Number of detected regimes:')
disp(Summary.nb_regime_per_param');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% How to forecast and compare with a model without breakpoints ? 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear arg_model;
[DGP] = DGP_simulation('AR','B'); %%% Simulation of a series from DGP 2 of the paper.
arg_model.AR_lags = 2; %% arg_model contains information about the model specification
[Summary,Simu] = SMC_CP_in_TS(DGP.y,'AR',arg_model,5000,950); %%% Estimation of a Sparse CP-AR(2) model and add one by one the obersarvations starting at t=950.
[Simu_no_break] = SMC_TS_no_break(DGP.y,'AR',arg_model,5000,950); %%% Estimation of a AR(2) model and add one by one the obersarvations starting at t=950.
MLL_pred = diff(Summary.MLL); %%% one step ahead log predictive density of the CP model over the 74 latest observations.
MLL_pred_no_break = diff(Simu_no_break.MLL); %%% one step ahead log predictive density of the model without breakpoints over the 74 latest observations.
figure;
plot(cumsum(MLL_pred-MLL_pred_no_break),'linewidth',3); %%% cumulative log predictive density over the 74 latest observations (one step ahead) --- positive values are in favor of the CP model. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% How to run ARMA models ?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear arg_model;
[DGP_ARMA] = DGP_simulation('ARMA','G'); %%% Simulation of a ARMA series from DGP 9 of the paper.
arg_model.AR_lags = 1; %% arg_model contains information about the model specification
arg_model.MA_lags = 1; %% arg_model contains information about the model specification
[Summary_ARMA,Simu_ARMA] = SMC_CP_in_TS(DGP_ARMA.y,'ARMA',arg_model,5000); %%% Estimation of a Sparse CP-ARMA(2,1) model

disp('\# of regimes for the intercept, AR term, MA term and the variance:')
disp(DGP_ARMA.nb_regime_per_param');
disp('Number of detected regimes:')
disp(Summary_ARMA.nb_regime_per_param');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% How to run a GARCH(1,1) model ?
%%% Not possible to change the model specification for the GARCH model.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear arg_model;
[DGP_GARCH] = DGP_simulation('GARCH','DGP12'); %%% DGP 12 of the paper.
[Summary_GARCH,Simu] = SMC_CP_in_TS(DGP_GARCH.y,'GARCH',[],5000); %%% Estimation of a Sparse CP-GARCH(1,1) model
disp(DGP_GARCH.nb_regime_per_param');
disp('Number of detected regimes:')
disp(Summary_GARCH.nb_regime_per_param');