function [Simu] = SMC_TS_no_break(y,type_model,arg_model,N_part,nb_for_SMC_sampler)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Sequential Monte Carlo sampler for simulating posterior
%%%%%%%%%%% distributions from AR, ARMA and GARCH models (without breakpoints).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Inputs and output similar to the code 'SMC_CP_in_TS.m'.
addpath('./2MU_functions');
addpath('./Statistical functions');
nb_prop_in_SADE = 10;
nCR_GARCH = 4;
taille = max(size(y));
if(taille~=size(y,1))
    y = y';
end
if(nargin<5)
    nb_for_SMC_sampler = taille;
end
error_type = 1;
switch type_model
    case 'AR'
        addpath('./Estimation_AR');
        if(isstruct(arg_model))
            if(isfield(arg_model,'AR_lags'))
                AR_lags = arg_model.AR_lags;
                disp(['Estimation of an AR(' num2str(AR_lags) ')'])
                intercept = 1;
                if(isfield(arg_model,'X'))
                    [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept,arg_model.X);
                else
                    [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept);
                end
                nb_param = size(DATA.X,2) + 1;
                taille = size(DATA.X,1);
                error_type = 0;
            end
        end
    case 'ARMA'
        addpath('./Estimation ARMA');
        if(isstruct(arg_model))
            if(isfield(arg_model,'MA_lags'))
                MA_lags = arg_model.MA_lags;
                if(isfield(arg_model,'AR_lags'))
                    AR_lags = arg_model.AR_lags;
                    disp(['Estimation of an ARMA(' num2str(AR_lags) ',' num2str(MA_lags) ')'])
                    intercept = 1;
                    if(isfield(arg_model,'X'))
                        [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept,arg_model.X);
                    else
                        [DATA] = AR_explanatory_var(y,1:AR_lags,[],intercept);
                    end
                    nb_param = size(DATA.X,2) + MA_lags + 1;
                    taille = size(DATA.X,1);
                    DATA.X = [DATA.X zeros(taille,MA_lags)];
                    error_type = 0;
                end
            end
        end
    case 'GARCH'
        addpath('./Estimation_GARCH');
        error_type = 0;
        DATA.y = y;
        DATA.X = zeros(max(size(y)),2);
        nb_param = 3;
        disp('Estimation of a GARCH(1,1)')
    case 'MSM'
        addpath('./MSM_with_leverage');
        if(isstruct(arg_model))
            if(isfield(arg_model,'kbar'))
                nb_param = 4;
                DATA.y = y;
                DATA.X = zeros(max(size(y)),2);
                error_type = 0;
            end
        end
end

if(error_type == 1)
    error('arg model is badly specified for the chosen model')
end

if(min(size(y))>1)
    error('The TS must be univariate')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization of the particles %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
log_like = zeros(N_part,1);
log_prior = zeros(N_part,1);
dim_model = nb_param;


nb_forecast = 1;
if(nb_for_SMC_sampler>taille)
    warning('nb for SMC should be smaller than the sample size'); %#ok<*WNTAG>
    nb_for_SMC_sampler = taille;
    nb_forecast = 1;
end
if(nb_forecast>1)
    warning('Above one step ahead, the forecast may be unreliable if AR or MA lags is different from zero');
end
up_to_time = nb_for_SMC_sampler;
y_t = DATA.y(1:up_to_time);
X_t = DATA.X(1:up_to_time,:);
if(up_to_time+nb_forecast>taille)
    X_for = zeros(nb_forecast,min(size(X_t)));
    y_for = zeros(nb_forecast,1);
else
    X_for = DATA.X(up_to_time+1:up_to_time+nb_forecast,:);
    y_for = DATA.y(up_to_time+1:up_to_time+nb_forecast);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Prior Varriances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
var_param = ones(dim_model,1);
switch type_model
    case{'GARCH','MSM'}
        var_param = 2*var_param;
    case{'AR','ARMA'}
        var_param(1:dim_model-1) = 10;
        var_param(end) = 1;
end


disp('***************************************')
disp('***************************************')
disp('**  Initialisation of the particles  **')
disp('***************************************')
disp('***************************************')
regime = 1;
particles = zeros(N_part,dim_model);
parfor q=1:N_part %%% Initialization of the particles - Drawing from the prior distributions
    count = 0;
    test = 0;
    while(test==0)
        log_theta = sqrt(var_param).*randn(dim_model,1);
        log_prior(q) = sum(-0.5*log(2*pi*var_param) - 0.5*log_theta.^2./var_param);
        particles(q,:) = log_theta';
        [log_like(q)] = launch_likelihood(log_theta,up_to_time,regime,y_t,X_t,type_model,arg_model);
        count = count +1;
        if(log_like(q)>-1e10)
            test = 1;
        end
        if(count>100)
            test = 1;
        end
    end
end




nb_all_iter = 2*taille;
Simu = struct('log_like',zeros(N_part,1),'MLL_all',zeros(nb_all_iter,1),'temper',zeros(nb_all_iter,1),'ESS',zeros(nb_all_iter,2),'accept_rate',zeros(nb_all_iter,nb_prop_in_SADE),'prob_type',zeros(nb_all_iter,nb_prop_in_SADE),'prob_CR',zeros(nb_all_iter,nCR_GARCH));

w_part = ones(N_part,1)/N_part; %% Initialisation of the wieghts of the particles.
MLL = 0;

i=2;
ESS_tab = zeros(nb_all_iter-1,2);
ESS_tab(1,:) = N_part;
SMC_end = 0; %% Notify when the SMC is finished.
sampler = 1; %% if = 1, iterating with SMC sampler otherwise with IBIS.
temper = 0; %% Initialisation of the temper function (phi(1) = 0)
val_iter = 0; %% counter for saving the posterior distribution.
nb_MCMC = 1;%nb_prop_in_SADE;%nb_prop_in_SADE*10; %%% Number of MCMC iterations at each SMC step.

ESS_borne = ceil(0.9*N_part); %%% Threshold which specifies when a resampling step takes place.
option = optimset('MaxIter',1e7,'MaxFunEvals',1e7);


post_log_like = zeros(N_part,nb_all_iter);
post_log_prior = zeros(N_part,nb_all_iter);
post_particles = zeros(N_part,dim_model,nb_all_iter);
post_time = zeros(nb_all_iter,1);
MLL_pred = zeros(nb_all_iter,1);
post_y_for = zeros(N_part,nb_forecast,nb_all_iter);
post_w = zeros(N_part,nb_all_iter);
post_pred_dens = zeros(nb_forecast,nb_all_iter);
post_pred_dens_simu = zeros(N_part,nb_forecast,nb_all_iter);







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialization of the Evolutionary algorithm %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
find_nb_chain = 20:130;
a = mod(N_part,find_nb_chain);
ind_chain = find(a==0,1);
if(isempty(ind_chain)==1)
    error('Wrong number of particles');
end
nb_chain = find_nb_chain(ind_chain); %%% Number of parallel chain into one MCMC of the Rejuvanation step.
ind_parfor = N_part/nb_chain; %%% Number of parallel MCMCs of the Rejuvanation step.

dens_stay_aide = zeros(nb_chain,ind_parfor);
prior_S_aide =  zeros(nb_chain,ind_parfor);
stay_particles_aide =  zeros(nb_chain,dim_model,ind_parfor);
stay_y_for_aide = zeros(nb_chain,nb_forecast,ind_parfor);
particles_y_for = zeros(N_part,nb_forecast);
stay_dens_pred_aide = zeros(nb_chain,nb_forecast,ind_parfor);
particles_dens_pred_for = zeros(N_part,nb_forecast);



adapt_rate_GARCH_DR = 1.5*ones(3,1);
ind_t = 1;
prob_type = ones(nb_prop_in_SADE,1)/nb_prop_in_SADE;
prob_CR_GARCH = ones(nCR_GARCH,1)/nCR_GARCH;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Initialisation of the Metropolis-Hasting proposal %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Cov_particles = zeros(dim_model,dim_model,ind_parfor);
while(SMC_end==0)
    if(sampler==1) %%% Tempering iteration - E-AIS step
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Correction step of the SMC sampler %%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Find the optimal value of phi(i) such that a specific
        %%%%%%%%%%%%% value of ESS is obtained.
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        ESS_target = floor(0.95*ESS_tab(i-1,1));
        val_prop = 1e-5:1e-4:0.1;
        ESS_control = calc_ESS(log_like,w_part,log(val_prop./(1-val_prop)),ESS_target);
        min_part_deviation = 0.05*N_part;
        aide = (ESS_control<min_part_deviation);
        if(sum(aide)>0)
            ind = find(aide==1);
            choice_diff = val_prop(ind(end));
        else
            %                 disp('Grid values did not work');
            val_opt = zeros(50,2);
            parfor q=1:50
                if(q<20)
                    val_dep_opt = rand()*0.005;
                else
                    val_dep_opt = rand()*0.35;
                end
                [val1,val2] = fminsearch(@(theta_phi) calc_ESS(log_like,w_part,theta_phi,ESS_target),log(val_dep_opt/(1-val_dep_opt)),option);
                val_opt(q,:) = [val1 val2];
            end
            [~,ind] = min(val_opt(:,2));
            log_choice_diff = val_opt(ind,1);
            choice_diff = exp(log_choice_diff)/(1+exp(log_choice_diff));
        end
        

        val_temp = temper;
        temper = min(temper + choice_diff,1);
        choice_diff = temper-val_temp;
        if(temper==1)
            sampler = 0;
        end
        w_aux = choice_diff*(log_like);
        [maxi] = max(w_aux);
        w_aux = w_part.*exp(w_aux-maxi);        
        Q = sum(w_aux);
        MLL = MLL + log(Q) + maxi;
        w_part = w_aux/Q;
    else
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Correction step of the IBIS or the TNT algorithm %%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%% Observations are added one by one.%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        up_to_time = up_to_time + 1;
        ind_t = ind_t +1;
        y_t = DATA.y(1:up_to_time);
        X_t = DATA.X(1:up_to_time,:);
        q_move = log_like;
        parfor z=1:N_part
            [log_like(z)] = launch_likelihood(particles(z,:)',up_to_time,regime,y_t,X_t,type_model,arg_model);
        end
        
        w_aux = log_like-q_move;
        [maxi] = max(w_aux);
        w_aux = w_part.*exp(w_aux-maxi);
        Q = sum(w_aux);
        
        
        MLL = MLL + log(Q) + maxi;
        w_part = w_aux/Q;
        if(up_to_time+nb_forecast>taille)
            X_for = zeros(nb_forecast,min(size(X_t)));
            y_for = zeros(nb_forecast,1);
        else
            X_for = DATA.X(up_to_time+1:up_to_time+nb_forecast,:);
            y_for = DATA.y(up_to_time+1:up_to_time+nb_forecast);
        end
    end
    
    
    Simu.temper(i-1) = temper;
    Simu.MLL_all(i-1) = MLL;
    
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Resampling using the stratified approach
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ESS = 1/sum(w_part.*w_part);
    if(ESS<ESS_borne)
        [indice] = multinomialrnd_c(w_part,rand(N_part,1));
        indice = indice(randperm(N_part));
        w_part = ones(N_part,1)/N_part;
        ESS = 1/sum(w_part.*w_part);
        
        aide_param = particles;
        log_like_aide = log_like;
        log_prior_aide = log_prior;
        log_like = log_like_aide(indice);
        log_prior = log_prior_aide(indice);
        particles = aide_param(indice,:);        
    end
    
    ESS_tab(i,1) = ESS;
    if(ESS_tab(i,2)==0)
        ESS_tab(i,2) = ESS;
    end
    disp('***************************************')
    disp('***************************************')
    disp(['Temper ' num2str(temper) ' and ESS ' num2str(ESS)])
    disp('***************************************')
    disp(['Nb MCMC : ' num2str(nb_MCMC)])
    disp('***************************************')
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Initialization of the Mutation step
    %%% by MH, DREAM and D-DREAM algorithms
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    accept_GARCH = zeros(ind_parfor,nb_prop_in_SADE);
    count_GARCH = zeros(ind_parfor,nb_prop_in_SADE);
    parfor q=1:ind_parfor
        ind = (q-1)*nb_chain;
        dens_stay_aide(:,q) = log_like(ind+1:ind+nb_chain); %#ok<*PFBNS>
        prior_S_aide(:,q) = log_prior(ind+1:ind+nb_chain);
        stay_particles_aide(:,:,q) = particles(ind+1:ind+nb_chain,:);
        Cov_particles(:,:,q) = cov(particles(ind+1:ind+nb_chain,:)) + 0.001*eye(dim_model);
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Mutation step : each living particle is mutated by an MCMC kernel
    %%% exhibiting as invariant distribution the current distribution of
    %%% the SMC
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Cov_particles_all_inv = inv(mean(Cov_particles,3));
    Mahalanobis_distance = zeros(ind_parfor,nb_prop_in_SADE);
    Mahalanobis_distance_CR_GARCH = zeros(ind_parfor,nCR_GARCH);
    count_CR_GARCH = zeros(ind_parfor,nCR_GARCH);
    accept_CR_GARCH = zeros(ind_parfor,nCR_GARCH);
    parfor q=1:ind_parfor
        dens_stay = dens_stay_aide(:,q);
        prior_S = prior_S_aide(:,q);
        stay_particles = stay_particles_aide(:,:,q)';
        stay_y_for = zeros(nb_forecast,nb_chain);
        stay_dens_pred = zeros(nb_forecast,nb_chain);
        accept_GARCH_parfor = ones(nb_prop_in_SADE,1);
        count_GARCH_parfor = ones(nb_prop_in_SADE,1);
        Mahalanobis_distance_parfor = zeros(nb_prop_in_SADE,1);
        Mahalanobis_distance_CR_GARCH_parfor = zeros(nCR_GARCH,1);
        count_CR_GARCH_parfor = zeros(nCR_GARCH,1);
        accept_CR_GARCH_parfor = zeros(nCR_GARCH,1);
        for M=1:nb_MCMC
            for DR=1:nb_chain

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%% Metropolis Move : SADE algorithm (Evolutionary)
                %%%% Update of all the model parameters in one block
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                choix_CR = multinomialrnd(prob_CR_GARCH,rand());
                count_CR_GARCH_parfor(choix_CR) = count_CR_GARCH_parfor(choix_CR) +1;
                [move_particles,log_dens_prop, selection] = SADE_final(stay_particles',dens_stay+prior_S,DR,(choix_CR-1)/nCR_GARCH,nb_chain,M,prob_type,adapt_rate_GARCH_DR);
                count_GARCH_parfor(selection) = count_GARCH_parfor(selection)+1;
                

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%% Computation of the prior densities
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%% Computation of the log-likelihood
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                prior_particles_move = sum(-0.5*log(2*pi*var_param) - 0.5*move_particles.^2./var_param);
                log_like_move = launch_likelihood(move_particles,up_to_time,regime,y_t,X_t,type_model,arg_model);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%% Computation of the Metropolis ratio
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                if(exp(temper*(log_like_move-dens_stay(DR))+prior_particles_move-prior_S(DR)+log_dens_prop)>rand())
                    dens_stay(DR) = log_like_move;
                    prior_S(DR) = prior_particles_move;
                    vect = stay_particles(:,DR)-move_particles;
                    dist = vect'*Cov_particles_all_inv*vect; %#ok<MINV>
                    Mahalanobis_distance_parfor(selection) = Mahalanobis_distance_parfor(selection) + dist; %% Computation of the Mahalanobis distance for the mutation equation
                    Mahalanobis_distance_CR_GARCH_parfor(choix_CR) = Mahalanobis_distance_CR_GARCH_parfor(choix_CR) + dist; %%% Computation of the Mahalanobis distance for the CR probability
                    stay_particles(:,DR) = move_particles;
                    accept_GARCH_parfor(selection) = accept_GARCH_parfor(selection)+1;
                    accept_CR_GARCH_parfor(choix_CR) = accept_CR_GARCH_parfor(choix_CR) +1;
                end     
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%% Forecasting
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                
                if(M==nb_MCMC && temper == 1)
                    [~,~,stay_y_for(:,DR),stay_dens_pred(:,DR)] = launch_likelihood(stay_particles(:,DR),up_to_time,regime,y_t,X_t,type_model,arg_model,X_for,y_for);
                end
            end
        end
        
        accept_GARCH(q,:) = accept_GARCH(q,:) + accept_GARCH_parfor';
        count_GARCH(q,:) = count_GARCH(q,:) + count_GARCH_parfor';
        stay_particles_aide(:,:,q) = stay_particles';
        
        
        dens_stay_aide(:,q) = dens_stay;
        prior_S_aide(:,q) = prior_S;
        stay_y_for_aide(:,:,q) = stay_y_for';
        stay_dens_pred_aide(:,:,q) = stay_dens_pred';
        
        Mahalanobis_distance(q,:) = Mahalanobis_distance(q,:) + Mahalanobis_distance_parfor';
        count_CR_GARCH(q,:) = count_CR_GARCH(q,:) + count_CR_GARCH_parfor';
        accept_CR_GARCH(q,:) = accept_CR_GARCH(q,:) + accept_CR_GARCH_parfor';
        Mahalanobis_distance_CR_GARCH(q,:) = Mahalanobis_distance_CR_GARCH(q,:) + Mahalanobis_distance_CR_GARCH_parfor';
        
    end
    Mahalanobis_distance = sum(Mahalanobis_distance,1) + 0.001;
    Mahalanobis_distance_CR_GARCH = sum(Mahalanobis_distance_CR_GARCH,1) + 0.001;
    count_GARCH = sum(count_GARCH,1);
    accept_GARCH = sum(accept_GARCH,1);
    count_CR_GARCH = sum(count_CR_GARCH,1);
    accept_CR_GARCH = sum(accept_CR_GARCH,1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Computation of the acceptance rates %%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    prob_type = (Mahalanobis_distance./max(count_GARCH,1))/(sum(Mahalanobis_distance./max(count_GARCH,1))); %% Update of the mutation probabilities
    prob_CR_GARCH = (Mahalanobis_distance_CR_GARCH./max(count_CR_GARCH,1))/(sum(Mahalanobis_distance_CR_GARCH./max(count_CR_GARCH,1))); %% Update of the CR probabilities
    
    rate_GARCH = accept_GARCH./count_GARCH;
    rate_CR_GARCH = accept_CR_GARCH./count_CR_GARCH;
    jump = (nb_prop_in_SADE-2)/2;
    adapt_rate_GARCH_DR(1) = max(adapt_rate_GARCH_DR(1) + (mean(rate_GARCH(1:jump))'-0.333)/(ind_t^(0.6)),1.01); %%% Adaption of Atchade and Rosenthal for targeting a fixed acceptance rate - For the Stretch mutations
    adapt_rate_GARCH_DR(2) = max(adapt_rate_GARCH_DR(2) + (mean(rate_GARCH(jump+1:2*jump))'-0.333)/(ind_t^(0.6)),1.01); %%% Adaption of Atchade and Rosenthal for targeting a fixed acceptance rate - For the Walk mutations
    adapt_rate_GARCH_DR(3) = max(adapt_rate_GARCH_DR(3) + (mean(rate_GARCH(2*jump+1:end))'-0.333)/(ind_t^(0.6)),0.01); %%% Adaption of Atchade and Rosenthal for targeting a fixed acceptance rate - For the DREAM mutations
    
    
    for q=1:ind_parfor
        ind = (q-1)*nb_chain;
        log_like(ind+1:ind+nb_chain) = dens_stay_aide(:,q);
        log_prior(ind+1:ind+nb_chain) = prior_S_aide(:,q);
        particles(ind+1:ind+nb_chain,:) = stay_particles_aide(:,:,q);
        particles_y_for(ind+1:ind+nb_chain,:) = stay_y_for_aide(:,:,q);
        particles_dens_pred_for(ind+1:ind+nb_chain,:) = stay_dens_pred_aide(:,:,q);
    end
    
    
    
    
    
    
    if(mod(i,1)==0)
        disp(['Fin de l iteration ' num2str(i) ' sur ' num2str(nb_all_iter)])
        disp(['Tempering ' num2str(temper) ' --- Time : ' num2str(up_to_time)])
        disp(['ML : ' num2str(max(log_like))])
        disp('*********************************************************')
        disp('********************** Accept Rate  *********************')
        disp('*********************************************************')
        disp(['Acceptation GARCH : ' num2str(rate_GARCH)])
        disp(['Acceptation CR GARCH : ' num2str(rate_CR_GARCH)])
        disp('*********************************************************')
        disp(['Adapt GARCH : ' num2str(adapt_rate_GARCH_DR')])
        disp('*********************************************************')
        disp(['Prob type GARCH : ' num2str(prob_type)])
        disp(['Prob CR GARCH : ' num2str(prob_CR_GARCH)])
        disp('*********************************************************')
    end
    

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%% Saving the draws and other quantities %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Simu.accept_rate(i-1,:) = rate_GARCH;
    Simu.prob_type(i-1,:) = prob_type;
    Simu.prob_CR(i-1,:) = prob_CR_GARCH;
    if(temper==1)
        val_iter = val_iter+1;
        MLL_pred(val_iter) = MLL;
        post_time(val_iter) = up_to_time;
        post_log_like(:,val_iter) = log_like;
        post_log_prior(:,val_iter) = log_prior;
        post_particles(:,:,val_iter) = particles;
        post_y_for(:,:,val_iter) = particles_y_for;
        post_pred_dens_simu(:,:,val_iter) = particles_dens_pred_for;
        post_w(:,val_iter) = w_part;      
        T_left = taille-up_to_time;
        if(T_left>nb_forecast)
            for t=1:nb_forecast
                post_pred_dens(t,val_iter) =  ksdensity(post_y_for(:,t,val_iter),DATA.y(up_to_time+t));
            end
        elseif(T_left>0)
            for t=1:T_left
                post_pred_dens(t,val_iter) =  ksdensity(post_y_for(:,t,val_iter),DATA.y(up_to_time+t));
            end
        end
    end
    i = i+1;
    if(i>nb_all_iter)
        disp('depassement du nombre iteration');
        i = nb_all_iter;
    end
    if(up_to_time==taille && temper==1)
        SMC_end = 1;
    end
end
Simu.count_GARCH = count_GARCH;
Simu.accept_GARCH = accept_GARCH;
Simu.ESS = ESS_tab;
total_iter = i-2;
Simu.MLL_all = Simu.MLL_all(1:total_iter);
Simu.temper = Simu.temper(1:total_iter);
Simu.ESS = Simu.ESS(1:total_iter-1,:);
Simu.accept_rate = Simu.accept_rate(1:total_iter-1,:);
Simu.prob_type = Simu.prob_type(1:total_iter-1,:);
Simu.prob_CR = Simu.prob_CR(1:total_iter-1,:);
Simu.adapt_rate_GARCH_DR = adapt_rate_GARCH_DR;

Simu.tau = post_time(1:val_iter);
Simu.log_like = post_log_like(:,1:val_iter);
Simu.log_prior = post_log_prior(:,1:val_iter);
Simu.log_theta = post_particles(:,:,1:val_iter);
Simu.y_for = post_y_for(:,:,1:val_iter);
Simu.w = post_w(:,1:val_iter);
Simu.post_pred_dens = post_pred_dens(:,1:val_iter);
Simu.post_pred_dens_simu = post_pred_dens_simu(:,:,1:val_iter);
Simu.MLL = MLL_pred(1:val_iter);
theta = zeros(N_part,dim_model);
parfor q=1:N_part %%% Initialization of the particles - Drawing from the prior distributions
    [~,param_current] = launch_likelihood(particles(q,:)',up_to_time,regime,y_t,X_t,type_model,arg_model);
    theta(q,:) = param_current;
end
Simu.theta = theta;
