function [Stat_Ng] = compute_NG_statistics_models(y,type_model,arg_model)
if(nargin<3)
    type_model = 'AR';
    arg_model.AR_lags = 1;
    warning('Not enough inputs: Estimation of an AR(1)');
end
error_model = 1;
switch type_model
    case 'AR'
        addpath('./Estimation_AR');
        if(isstruct(arg_model))
            if(isfield(arg_model,'AR_lags'))
                AR_lags = arg_model.AR_lags;
                MA_lags = 0;
                disp(['Estimation of an AR(' num2str(AR_lags) ')'])
                intercept = 1;
                [DATA] = AR_explanatory_var(y,AR_lags,[],intercept);
                nb_param = AR_lags + 2;
                error_model = 0;
            end
        end
    case 'ARMA'
        addpath('./Estimation ARMA');
        if(isstruct(arg_model))
            if(isfield(arg_model,'MA_lags'))
                MA_lags = arg_model.MA_lags;
                if(isfield(arg_model,'AR_lags'))
                    AR_lags = arg_model.AR_lags;
                    disp(['Estimation of an ARMA(' num2str(AR_lags) ',' num2str(MA_lags) ')'])
                    intercept = 1;
                    [DATA] = AR_explanatory_var(y,AR_lags,[],intercept);
                    nb_param = AR_lags + MA_lags + 2;
                    error_model = 0;
                end
            end
        end
    case 'GARCH'
        addpath('./Estimation_GARCH');
        error_model = 0;
        DATA.y = y;
        DATA.X = zeros(max(size(y)),2);
        MA_lags = 0;
        nb_param = 3;
        disp('Estimation of a GARCH(1,1)')
end
if(error_model == 1)
    error('arg model is badly specified for the chosen model')
end

if(min(size(y))>1)
    error('The TS must be univariate')
end

T = max(size(DATA.y));
if(strcmp(type_model,'AR')==1)
    h = round(2*log(T)^2); %%% Constant fixed in Yau (2016) %round(max(100,log(T)^4/25));
else
    h = round(max(100,log(T)^4/25));
end
h1 = round(h/4);
LLs = zeros(T,3);
Stats = zeros(T,1);
if(h<T)
    parfor t=1:T
        %     current_y = 0;
        if(t>h && t<T-h)
            current_y = DATA.y(t-h+1:t+h); %#ok<*PFBNS>
            current_X = DATA.X(t-h+1:t+h,:);
            step = h;
            switch type_model
                case 'AR'
                    [result1] = OLS_estimation_ARX(current_y(1:step),current_X(1:step,:));
                    [result2] = OLS_estimation_ARX(current_y(step+1:end),current_X(step+1:end,:));
                    [result3] = OLS_estimation_ARX(current_y,current_X);
                    
                case 'ARMA'
                    [result1] = max_ARMA(current_y(1:step),current_X(1:step,:),MA_lags);
                    [result2] = max_ARMA(current_y(step+1:end),current_X(step+1:end,:),MA_lags);
                    [result3] = max_ARMA(current_y,current_X,MA_lags);
                case 'GARCH'
                    [result1] = MLE_GARCH_with_known_break_dates(current_y(1:step),max(size(current_y(1:step))),1);
                    [result2] = MLE_GARCH_with_known_break_dates(current_y(step+1:end),max(size(current_y(step+1:end))),1);
                    [result3] = MLE_GARCH_with_known_break_dates(current_y,max(size(current_y)),1);
            end
            LLs(t,:) = [result1.log_like result2.log_like result3.log_like];
            Stats(t) = (result1.log_like +result2.log_like - result3.log_like)/(2*step);
        end
    end
end

tau_all = zeros(T,2);
if(strcmp(type_model,'AR')==1)
    h2 = round(0.5*h);
else
    h2 = round(0.75*h);
end

for t=1:T
    if(t>h && t<T-h)
        [maxi,~] = max(Stats(t-h:t+h));
        [maxi2,~] = max(Stats(t-h2:t+h2));
        if(maxi==Stats(t) && maxi~=0)
            tau_all(t,1) = t;
        end
        if(maxi2==Stats(t) && maxi2~=0)
            tau_all(t,2) = t;
        end
    end
end
J = tau_all(tau_all(:,1)~=0,1);
J1 = tau_all(tau_all(:,2)~=0,2);

Stat_Ng.J = J;
Stat_Ng.J1 = J1;
Stat_Ng.Stats = Stats;
Stat_Ng.LLs = LLs;
Stat_Ng.h = h;
% Kmax = size(Stat_Ng.J1,1) + 2;
if(strcmp(type_model,'GARCH')==1)
    Kmax = size(Stat_Ng.J,1);
    if(Kmax == 0)
        Kmax = 1;
    end
    Ubounds = ones(Kmax,2);
    
    J2 = Stat_Ng.J;
    [sorted] = sort(J2);
    if(sum(sorted==J2)==max(size(J2)))
        min_duration = 2*nb_param;
        if(T-h1-h1<round(min_duration))
            Ubounds = [ceil(0.5*h1) ceil(0.5*(T+h1))];
        else
            a = round((J2(1:end-1) + J2(2:end))/2);
            Ubounds(1:end-1,2) = a-round(min_duration/2);
            Ubounds(2:end,1) = a+round(min_duration/2);
        end
        Ubounds(end,2) = T-h1;
    else
        Ubounds = [1 T];
    end
else
    Kmax = size(Stat_Ng.J1,1) + 2;
    Ubounds = ones(Kmax,2);
    Ubounds(1,1) = h1;
    Ubounds(end,2) = T-h1;
    J2 = [h1;J1;T-h1];
    [sorted] = sort(J2);
    if(sum(sorted==J2)==max(size(J2)))
        min_duration = 2*nb_param;
        if(T-h1-h1<round(min_duration))
            Ubounds = [ceil(0.5*h1) ceil(0.5*(T+h1))];
        else
            a = round((J2(1:end-1) + J2(2:end))/2);
            Ubounds(1:end-1,2) = a-round(min_duration/2);
            Ubounds(2:end,1) = a+round(min_duration/2);
        end
        Ubounds(end,2) = T-h1;
    else
        Ubounds = [1 T];
    end
end
Stat_Ng.Ubounds = Ubounds;





