function [Simu] = find_best_configuration(Simu)
%%%% Produces summary of the posterior draws
simu_over_time = 0;
if(size(size(Simu.break_dates),2)==3)
    simu_over_time = 1;
end

N = size(Simu.break_dates,1);
nb_reg = min(size(Simu.break_dates)) + 1;
nb_param = min(size(Simu.log_theta_1reg));


all_conf = zeros(N,nb_param);
for i=1:N
    a = squeeze(Simu.param_in_small_comp(:,:,i)); 
    nb_reg_current = sum(a==0,2)+1;
    all_conf(i,:) = nb_reg_current';
end
nb_max_reg = max(max(all_conf));
prob_regime_per_param = zeros(nb_param,nb_max_reg);
for i=1:nb_param
    for j=1:nb_max_reg
        prob_regime_per_param(i,j) = sum(all_conf(:,i)==j)/N;
    end
end

ind_conf = 0;
ind_poss = ones(N,1);
conf_poss = zeros(N,nb_param);
count_poss = zeros(N,1);
for i=1:N
    if(ind_poss(i)~=0)
        a = kron(ones(N,1),all_conf(i,:));
        ind = (sum(((a-all_conf)==0),2)==nb_param);
        count = sum(ind);
        if(count~=0)
            ind_conf  = ind_conf +1;
            conf_poss(ind_conf,:) = all_conf(i,:);
            ind_poss(ind) = 0;
            count_poss(ind_conf) = count;
        end
    end
end

post_conf = conf_poss(1:ind_conf,:);
post_prob_conf = count_poss(1:ind_conf);
post_prob_conf = post_prob_conf/sum(post_prob_conf);

[sorted,ind_sorted] = sort(post_prob_conf,'descend');
post_prob_conf = sorted;
post_conf = post_conf(ind_sorted,:);

Simu.post_conf = post_conf;
Simu.post_prob_conf = post_prob_conf;



best_conf = post_conf(1,:);
a = kron(ones(N,1),best_conf);
ind = (sum((a-all_conf==0),2)==nb_param);
if(simu_over_time==1)
    break_sampled = Simu.break_dates(ind,:,end);
else
    break_sampled = Simu.break_dates(ind,:);
end
T = max(size(Simu.param_t));
post_mean_cass = [1 mean(break_sampled) T];
val_mean_cass = round(0.5*(post_mean_cass(1:end-1)+post_mean_cass(2:end)));
theta_t = Simu.theta(ind,:,:);

nb_best_conf = sum(ind);
nb_param = size(Simu.log_theta_1reg,2);
param_t = zeros(nb_param,max(best_conf),nb_best_conf);
break_sampled = zeros(max(best_conf),nb_best_conf,nb_param);
ind_count = 0;
for i=1:N
    if(ind(i)==1)
        ind_count = ind_count + 1;
        a = (Simu.param_in_small_comp(:,:,i)==0);
        for z=1:nb_param
            if(sum(a(z,:))~=0)
                if(simu_over_time==1)
                    aide = Simu.break_dates(i,a(z,:),end)';
                else
                    aide = Simu.break_dates(i,a(z,:))';
                end
                taille_reg = max(size(aide));
                break_sampled(1:taille_reg,ind_count,z) = aide;
            else
                taille_reg = 0;
            end
            param_t(z,1:taille_reg+1,ind_count) = squeeze(Simu.theta(i,logical([1 a(z,:)]),z))';
        end
    end
end

quant = [0.1:0.1:0.9];
nb_quant = max(size(quant));
post_dist_param = zeros(nb_param,max(best_conf),nb_quant);
post_dist_break = zeros(nb_param,max(best_conf),nb_quant);
for q=1:nb_quant
    post_dist_param(:,:,q) = squeeze(quantile(param_t,quant(q),3));
    post_dist_break(:,:,q) = squeeze(quantile(break_sampled,quant(q),2))';
end


summary_best_conf.post_dist_param = post_dist_param;
summary_best_conf.post_dist_break = post_dist_break;
summary_best_conf.post_conf =  post_conf;
summary_best_conf.post_prob_conf =  post_prob_conf;
summary_best_conf.best_conf = best_conf;
summary_best_conf.prob_best_conf = post_prob_conf(1);

Simu.summary_best_conf = summary_best_conf;
Simu.prob_regime_per_param = prob_regime_per_param;



