function [log_like,theta,y_for,pred_dens,epsilon] = launch_likelihood(log_theta,break_dates,regime,y,X,type_model,arg_model,X_forecast,y_true)
%%%%%%%%%%%%%%%%%%%%%
%%% Evaluate the log-likelihood function at log_theta with breakpoint
%%% locations given by break_dates. 
%%%%%%%%%%%%%%%%%%%%%%
%%% This function is not intended to be used outside the functions
%%% SMC_CP_in_TS.m
%%%%%%%%%%%%%%%%%%%%%%
do_pred_density = 1;
do_forecast = 1;
y_for = 0;
pred_dens = 0;
epsilon = 0;
if(nargin<9)
    y_true = 0;
    do_pred_density = 0;
    if(nargin<8)
        do_forecast = 0;
    else
        y_for = zeros(size(X_forecast,1),1);
    end
elseif(size(X_forecast,1)~=size(y_true,1))
    y_true = 0;
    do_pred_density = 0;
end
if(do_pred_density==1)
    pred_dens = zeros(size(X_forecast,1),1);
end
error_type = 1;

switch type_model
    case 'AR'
        if(isstruct(arg_model))
            if(isfield(arg_model,'AR_lags'))
                MA_lags = 0;
                nb_param = size(X,2) + 1;
                error_type = 0;
            end
        end
    case 'ARMA'
        if(isstruct(arg_model))
            if(isfield(arg_model,'MA_lags'))
                MA_lags = arg_model.MA_lags;
                if(isfield(arg_model,'AR_lags'))
                    nb_param = size(X,2) + 1;
                    error_type = 0;
                end
            end
        end
    case 'GARCH'
        nb_param = 3;
        error_type = 0;
        do_forecast = 0;
end
if(error_type == 1)
    error('arg model is badly specified for the chosen model')
end

if(regime==1)
    switch type_model
        case {'AR','ARMA'}
            sigma_current = exp(log_theta(end));
            theta_current = log_theta(1:end-1);
            taille_ARMA = size(X,2); 
            spike_and_slab = 0;
            theta = [theta_current;sigma_current];
            [log_like,~,~,~,X_all]= sorting_variables(theta_current,sigma_current,break_dates,regime,taille_ARMA,MA_lags,y,X,spike_and_slab);
            theta_all_reg = theta;
            epsilon = X_all;
        case 'GARCH'
            theta = log_theta;            
            theta(1) = exp(log_theta(1));
            theta(3) = exp(log_theta(3))/(1+exp(log_theta(3)));
            theta(2) = 0.5*exp(log_theta(2))/(1+exp(log_theta(2)));
            mu = 0;
            unc_var = theta(1);
            [log_like] = dens_CP_GARCH_MCMC(y,theta,mu,max(size(y)),unc_var);
            epsilon = y;
    end
else
    Kmax = regime - 1;
    if(size(break_dates,1)==Kmax)
        break_dates = [break_dates;max(size(y))];
    end
    
    log_theta_1reg = log_theta(1:nb_param);
    log_theta_2MU = log_theta(nb_param+1:end);
    switch type_model
        case {'AR','ARMA'}
            
            
            [theta_all_reg] = transform_2MU_into_model_parameters(log_theta_1reg,log_theta_2MU,type_model,nb_param,Kmax);
            
            theta_all_reg(end,:) = exp(theta_all_reg(end,:));
            
            taille_ARMA = size(X,2);
            spike_and_slab = 0;
            theta_current = reshape(theta_all_reg(1:nb_param-1,:),regime*(nb_param-1),1);
            sigma_current = theta_all_reg(end,:)';
            [log_like,~,~,~,X_all]= sorting_variables(theta_current,sigma_current,break_dates,regime,taille_ARMA,MA_lags,y,X,spike_and_slab);
            epsilon = X_all;
        case 'GARCH'

            [log_theta_all_reg] = transform_2MU_into_model_parameters(log_theta_1reg,log_theta_2MU,type_model,nb_param,Kmax);           
            log_theta_all_reg(1,:) = exp(log_theta_all_reg(1,:));
            log_theta_all_reg(3,:) = exp(log_theta_all_reg(3,:))./(1+exp(log_theta_all_reg(3,:)));
            log_theta_all_reg(2,:) = 0.5*exp(log_theta_all_reg(2,:))./(1+exp(log_theta_all_reg(2,:)));
            theta_all_reg = log_theta_all_reg;
            mu = zeros(regime,1);
            theta_current = reshape(log_theta_all_reg,regime*nb_param,1);
            unc_var = theta_current(1);
            [log_like] = dens_CP_GARCH_MCMC(y,theta_current,mu,break_dates,unc_var);
            epsilon = y;
    end
    theta = theta_all_reg;
end

%%%% Forecasting if AR or ARMA.
if(do_forecast==1)
    nb_forecast = size(X_forecast,1);
    y_for = zeros(nb_forecast,1);
    dim = size(X_forecast,2)-MA_lags;
    if(taille_ARMA-MA_lags~=dim)
        error('Probleme dimension forecast');
    end
    X_for = zeros(1,taille_ARMA);
    if(MA_lags>0)
        X_for(dim+1:end) = X_all(end:-1:end-MA_lags+1);
    end
    current_regime = sum(break_dates<break_dates(end)) +1;
    epsilon_for = sqrt(sigma_current(current_regime))*randn(nb_forecast,1);
    for t=1:nb_forecast
        X_for(1:dim) = X_forecast(t,1:dim);
        y_for(t) = X_for*theta_all_reg(1:end-1,current_regime) + epsilon_for(t);
        if(do_pred_density==1)
            pred_dens(t) = exp(-0.5*log(2*pi*sigma_current(current_regime)) -0.5*(y_true(t)-X_for*theta_all_reg(1:end-1,current_regime))^2/(sigma_current(current_regime))); %#ok<AGROW>
        end
        if(MA_lags>0 && MA_lags>=t)
            if(MA_lags>1)
                X_for(dim+2:end) = X_for(dim+1:end-1);
            end
            X_for(dim+1) = epsilon_for(t);
            
        end
        
    end
    
end